#!/usr/bin/env python

from .field import AttrField, BaseField, TextField
from .item import Item
from .middleware import Middleware
from .request import Request
from .spider import Spider

__version__ = "0.0.5"
